package com.doc.biz.service.impl;

import com.alibaba.fastjson2.JSONObject;
import com.doc.biz.domain.DocDir;
import com.doc.biz.domain.DocDirLevel;
import com.doc.biz.domain.DocFavorite;
import com.doc.biz.domain.DocSpace;
import com.doc.biz.mapper.DocDirMapper;
import com.doc.biz.service.IDocDirLevelService;
import com.doc.biz.service.IDocDirService;
import com.doc.biz.service.IDocFavoriteService;
import com.doc.biz.service.IDocSpaceService;
import com.doc.common.core.domain.entity.SysRole;
import com.doc.common.utils.DateUtils;
import com.doc.common.utils.SecurityUtils;
import com.doc.common.utils.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 文档目录管理Service业务层处理
 *
 * @author wukai
 * @date 2023-08-15
 */
@Service
public class DocDirServiceImpl implements IDocDirService {
    @Resource
    private DocDirMapper docDirMapper;
    @Resource
    private IDocSpaceService docSpaceService;
    @Resource
    private IDocFavoriteService favoriteService;
    @Resource
    private IDocDirLevelService levelService;

    /**
     * 查询文档目录管理
     *
     * @param dirId 文档目录管理主键
     * @return 文档目录管理
     */
    @Override
    public DocDir selectDocDirByDirId(Long dirId) {
        return docDirMapper.selectDocDirByDirId(dirId);
    }

    /**
     * 查询目录是否有某个权限
     *
     * @param dirId    文档目录管理主键
     * @param roleCode 权限编码
     * @return 文档目录管理
     */
    @Override
    public Boolean hasRole(Long dirId, String roleCode) {
        DocDir dir = selectDocDirByDirId(dirId);
        String yes = "Y";
        if (yes.equals(dir.getIsEncrypt())) {
            DocDirLevel level = levelService.selectDocDirLevelByLevelCode(dir.getEncryptLevel());
            JSONObject obj = JSONObject.parse(level.getLevelRoles());
            if (obj.getInteger(roleCode) == null || obj.getInteger(roleCode) != 1) {
                //只有权限为选中状态才有滴
                return false;
            }
        }
        //如果不是加密目录，肯定是所有权限都有滴
        return true;
    }

    /**
     * 验证访问码
     *
     * @param dirId      文档目录管理主键
     * @param accessCode SM3加密之后的访问码
     * @return 文档目录管理
     */
    @Override
    public boolean matchesCode(Long dirId, String accessCode) {
        DocDir dir = docDirMapper.selectById(dirId);
        return SecurityUtils.matchesPassword(accessCode, dir.getAccessCode());
    }

    /**
     * 查询文档目录管理列表
     *
     * @param docDir 文档目录管理
     * @return 文档目录管理
     */
    @Override
    public List<DocDir> selectDocDirList(DocDir docDir) {
        List<DocDir> list = docDirMapper.selectDocDirList(docDir);

        DocFavorite favorite = new DocFavorite();
        favorite.setOwner(SecurityUtils.getUserId());
        favorite.setIsFolder("Y");

        List<Long> favorites = new ArrayList<>();
        favoriteService.selectDocFavoriteList(favorite).forEach(f -> {
            favorites.add(f.getRelaId());
        });

        list.forEach(d -> {
            if (favorites.contains(d.getDirId())) {
                d.setIsFavorite("Y");
            } else {
                d.setIsFavorite("N");
            }
        });
        return list;
    }

    /**
     * 新增文档目录管理
     *
     * @param docDir 文档目录管理
     * @return 结果
     */
    @Override
    public int insertDocDir(DocDir docDir) {
        docDir.setCreateTime(DateUtils.getNowDate());
        //如果设置了访问码，则需要加密存储
        if (StringUtils.isNotEmpty(docDir.getAccessCode())) {
            String pass = SecurityUtils.encryptPassword(docDir.getAccessCode());
            docDir.setAccessCode(pass);
        }

        return docDirMapper.insertDocDir(docDir);
    }


    /**
     * 修改文档目录管理
     *
     * @param docDir 文档目录管理
     * @return 结果
     */
    @Override
    public int updateDocDir(DocDir docDir) {
        docDir.setUpdateTime(DateUtils.getNowDate());
        return docDirMapper.updateDocDir(docDir);
    }

    /**
     * 批量删除文档目录管理
     *
     * @param dirIds 需要删除的文档目录管理主键
     * @return 结果
     */
    @Override
    public int deleteDocDirByDirIds(Long[] dirIds) {
        return docDirMapper.deleteDocDirByDirIds(dirIds);
    }

    /**
     * 删除文档目录管理信息
     *
     * @param dirId 文档目录管理主键
     * @return 结果
     */
    @Override
    public int deleteDocDirByDirId(Long dirId) {
        return docDirMapper.deleteDocDirByDirId(dirId);
    }

    /**
     * 初始化顶层目录
     *
     * @param space 空间信息
     */
    @Override
    public void initTopDir(DocSpace space) {
        //初始化顶层目录
        DocDir dir = new DocDir();
        dir.setSpaceId(space.getSpaceId());
        dir.setDirName(space.getSpaceName());
        dir.setParentId(0L);
        List<DocDir> dirList = selectDocDirList(dir);
        if (dirList.size() > 0) {
            //如果有记录，则更新顶层目录
            dir.setDirId(dirList.get(0).getDirId());
            updateDocDir(dir);
        } else {
            //如果没有，则插入
            insertDocDir(dir);
        }
    }

    /**
     * 查询当前用户下所有有权限的目录
     *
     * @param userId 用户ID
     * @return 结果集
     */
    @Override
    public Map<String, Object> selectDirByUser(Long userId, String type) {
        Map<String, Object> result = new HashMap<>(4);
        List<DocSpace> spaceList = docSpaceService.getSpaceByUser(userId);
        result.put("space", spaceList);
        List<Long> spaceIds = new ArrayList<>();
        List<DocDir> dirs = new ArrayList<>();
        spaceList.forEach(space -> {
            if (StringUtils.isEmpty(type) || type.equals(space.getSpaceType())) {
                spaceIds.add(space.getSpaceId());

                DocDir dir = new DocDir();
                dir.setSpaceId(space.getSpaceId());

                addRole(dir, space.getSpaceType());
                dirs.addAll(docDirMapper.selectDocDirList(dir));

            }
        });
        result.put("dir", dirs);
        return result;
    }


    /**
     * 添加公共目录和部门目录权限控制
     *
     * @param dir  目录
     * @param type 空间类型
     */
    @Override
    public void addRole(DocDir dir, String type) {
        String pubKey = "1";
        String deptKey = "2";
        if (!pubKey.equals(type) && !deptKey.equals(type)) {
            //如果是个人目录，则直接返回
            return;
        }
        String pubRole = "system";
        String deptRole = "dept";
        //如果不是个人空间，则需要按权限
        List<SysRole> roles = SecurityUtils.getLoginUser().getUser().getRoles();
        Set<String> roleKeys = roles.stream().map(SysRole::getRoleKey).collect(Collectors.toSet());
        //如果是公共空间，且不为系统管理员
        boolean pubFlag = pubKey.equals(type) && !roleKeys.contains(pubRole);
        //如果是部门空间，且不为部门管理员
        boolean deptFlag = deptKey.equals(type) && !roleKeys.contains(deptRole);
        if (pubFlag || deptFlag) {
            //则需要判断组织目录是否有权限访问
            Map<String, Object> params = new HashMap<>(4);
            params.put("flag", 1);
            params.put("uid", SecurityUtils.getUserId());
            dir.setParams(params);
        }
    }
}
