package com.doc.biz.service.impl;

import cn.hutool.core.io.IoUtil;
import com.doc.biz.domain.DocumentFile;
import com.doc.biz.mapper.MongoFileRepository;
import com.doc.biz.service.IMongoService;
import com.doc.biz.vo.DocumentVO;
import com.doc.common.constant.Constants;
import com.doc.common.utils.file.FileUtils;
import com.doc.common.utils.sign.Md5Utils;
import com.mongodb.client.gridfs.GridFSBucket;
import com.mongodb.client.gridfs.GridFSDownloadStream;
import lombok.extern.slf4j.Slf4j;
import org.bson.types.Binary;
import org.bson.types.ObjectId;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.gridfs.GridFsTemplate;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import org.yaml.snakeyaml.util.UriEncoder;

import javax.annotation.Resource;
import java.io.InputStream;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * 文档协作Service业务层处理
 *
 * @author wukai
 * @date 2023-08-15
 */
@Slf4j
@Service
public class MongoServiceImpl implements IMongoService {

    @Resource
    private MongoFileRepository mongoFileRepository;
    @Resource
    private MongoTemplate mongoTemplate;
    @Resource
    private GridFsTemplate gridFsTemplate;
    @Resource
    private GridFSBucket gridFSBucket;

    /**
     * 多文件上传
     *
     * @param files 文件列表
     * @return list
     */
    @Override
    public List<DocumentVO> uploadFiles(List<MultipartFile> files) {
        return files.stream().map(file -> {
            try {
                return this.uploadFile(file);
            } catch (Exception e) {
                log.error("文件上传失败", e);
                return null;
            }
        }).filter(Objects::nonNull).collect(Collectors.toList());
    }

    /**
     * 文件上传
     *
     * @param file 文件
     * @return file
     */
    @Override
    public DocumentVO uploadFile(MultipartFile file) throws Exception {
        if (file.getSize() > Constants.MONGO_FILE_SIZE) {
            return this.saveGridFsFile(file);
        } else {
            return this.saveBinaryFile(file);
        }
    }

    /**
     * 文件下载
     *
     * @param fileId 文件ID
     * @return 文档
     */
    @Override
    public DocumentVO downloadFile(String fileId) {
        Optional<DocumentFile> option = this.findByFileId(fileId);

        if (option.isPresent()) {
            DocumentFile documentFile = option.get();
            if (Objects.isNull(documentFile.getContent())) {
                option = this.getGridFsFileById(fileId);
            }
        }

        return option.map(DocumentVO::new).orElse(null);
    }

    /**
     * 前端文件下载
     *
     * @param fileId 文件ID
     * @param isDown 是否下载
     * @return 下载流
     */
    @Override
    public ResponseEntity<Object> download(String fileId, boolean isDown) {
        DocumentVO vo = downloadFile(fileId);
        if (Objects.nonNull(vo)) {
            String disposition = isDown ? "attachment" : "inline";
            disposition += "; filename=\"" + UriEncoder.encode(vo.getFileName()) + "\"";
            return ResponseEntity.ok()
                    .header(HttpHeaders.CONTENT_DISPOSITION, disposition)
                    .header(HttpHeaders.CONTENT_TYPE, vo.getContentType())
                    .header(HttpHeaders.CONTENT_LENGTH, vo.getFileSize() + "")
                    .header("Connection", "close")
                    .body(vo.getData());
        } else {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body("file does not exist");
        }
    }

    /**
     * 文件删除
     *
     * @param fileId 文件ID
     */
    @Override
    public void removeFile(String fileId) {
        Optional<DocumentFile> option = this.findByFileId(fileId);
        if (option.isPresent()) {
            if (Objects.nonNull(option.get().getGridFsId())) {
                this.removeGridFsFile(fileId);
            } else {
                this.removeBinaryFile(fileId);
            }
        }
    }

    /**
     * 删除Binary文件
     *
     * @param fileId 文件ID
     */
    public void removeBinaryFile(String fileId) {
        mongoFileRepository.deleteById(fileId);
    }

    /**
     * 删除GridFs文件
     *
     * @param fileId
     */
    public void removeGridFsFile(String fileId) {
        // 根据id查询文件
        DocumentFile documentFile = mongoTemplate.findById(fileId, DocumentFile.class);
        if (Objects.nonNull(documentFile)) {
            // 根据文件ID删除fs.files和fs.chunks中的记录
            Query deleteFileQuery = new Query().addCriteria(Criteria.where("_id").is(documentFile.getGridFsId()));
            gridFsTemplate.delete(deleteFileQuery);
            // 删除集合documentFile中的数据
            Query deleteQuery = new Query(Criteria.where("id").is(fileId));
            mongoTemplate.remove(deleteQuery, DocumentFile.class);
        }
    }

    /**
     * 保存Binary文件（小文件）
     *
     * @param file
     * @return
     * @throws Exception
     */
    public DocumentVO saveBinaryFile(MultipartFile file) throws Exception {
        String suffix = FileUtils.getFileSuffix(file);
        try (InputStream in = file.getInputStream()) {
            DocumentFile documentFile = mongoFileRepository.save(
                    DocumentFile.builder()
                            .fileName(file.getOriginalFilename())
                            .fileSize(file.getSize())
                            .content(new Binary(file.getBytes()))
                            .contentType(file.getContentType())
                            .uploadDate(new Date())
                            .suffix(suffix)
                            .md5(Md5Utils.getMd5ByInputStream(in))
                            .build()
            );
            return new DocumentVO(documentFile);
        } catch (Exception e) {
            log.error(e.getMessage());
            throw e;
            // 抛出异常，让调用方处理
        }

    }

    /**
     * 保存GridFs文件（大文件）
     *
     * @param file 文件
     * @return vo
     */
    public DocumentVO saveGridFsFile(MultipartFile file) throws Exception {
        String suffix = FileUtils.getFileSuffix(file);
        try (InputStream in = file.getInputStream()) {
            String gridFsId = this.storeFileToGridFS(in, file.getOriginalFilename(), file.getContentType());
            DocumentFile documentFile = mongoFileRepository.save(
                    DocumentFile.builder()
                            .fileName(file.getOriginalFilename())
                            .fileSize(file.getSize())
                            .contentType(file.getContentType())
                            .uploadDate(new Date())
                            .suffix(suffix)
                            .md5(Md5Utils.getMd5ByInputStream(in))
                            .gridFsId(gridFsId)
                            .build()
            );
            return new DocumentVO(documentFile);
        } catch (Exception e) {
            log.error(e.getMessage());
            throw e;
            // 抛出异常，让调用方处理
        }
    }

    /**
     * 上传文件到Mongodb的GridFs中
     *
     * @param in          文件输入流
     * @param contentType type
     * @return id
     */
    public String storeFileToGridFS(InputStream in, String filename, String contentType) {
        // 将文件存储进GridFS中
        ObjectId gridFsId = gridFsTemplate.store(in, filename, contentType);
        return gridFsId.toString();
    }

    /**
     * 根据文件ID获取文档内容
     *
     * @param fileId 文件ID
     * @return documentFile
     */
    @Override
    public Optional<DocumentFile> findByFileId(String fileId) {
        return mongoFileRepository.findById(fileId);
    }

    /**
     * 查询并复制mongo文件
     *
     * @param id 文件id
     * @return 对象
     */
    @Override
    public DocumentVO copy(String id) {
        Optional<DocumentFile> optional = mongoFileRepository.findById(id);
        if (optional.isPresent()) {
            DocumentFile file = optional.get();
            file.setId(null);
            mongoFileRepository.save(file);
            return new DocumentVO(file);
        } else {
            return null;
        }
    }

    /**
     * 获取Grid文件
     *
     * @param id 文件ID
     * @return 文档
     */
    public Optional<DocumentFile> getGridFsFileById(String id) {
        DocumentFile documentFile = mongoTemplate.findById(id, DocumentFile.class);
        if (Objects.nonNull(documentFile)) {
            try (GridFSDownloadStream in = gridFSBucket.openDownloadStream(new ObjectId(documentFile.getGridFsId()))) {
                documentFile.setContent(new Binary(IoUtil.readBytes(in)));
                return Optional.of(documentFile);
            } catch (Exception e) {
                log.error("获取MongoDB大文件失败", e);
            }
        }

        return Optional.empty();
    }
}
