package com.doc.common.utils;

import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.text.PDFTextStripper;
import org.apache.poi.hslf.usermodel.HSLFSlideShow;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.hwpf.extractor.WordExtractor;
import org.apache.poi.sl.extractor.SlideShowExtractor;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xslf.usermodel.XMLSlideShow;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.apache.poi.xwpf.extractor.XWPFWordExtractor;
import org.apache.poi.xwpf.usermodel.XWPFDocument;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;

/**
 * 读取文件内容
 * WPS WORD TXT
 *
 * @author wukai
 */
public class FileContentUtils {

    /**
     * 获取正文文件内容，docx方法
     *
     * @param data 二进制文件内容
     * @return
     */
    public static String getContentDocx(byte[] data) {
        // 0表示获取正常，1表示获取异常
        try (InputStream is = new ByteArrayInputStream(data)) {
            //根据需求入参也可以改为文件路径，对应的输入流部分改为new File(路径)即可
            // 2007版本的word
            XWPFDocument xwpf = new XWPFDocument(is);
            XWPFWordExtractor extractor = new XWPFWordExtractor(xwpf);
            return extractor.getText();
        } catch (Exception e) {
            e.printStackTrace();
            return "";
        }
    }

    /**
     * 获取正文文件内容，doc方法
     *
     * @param data 二进制文件内容
     * @return
     */
    public static String getContentDoc(byte[] data) {
        try (InputStream is = new ByteArrayInputStream(data)) {
            // 2003版本的word
            WordExtractor wordExtractor = new WordExtractor(is);
            return wordExtractor.getText();
        } catch (IllegalArgumentException e) {
            //做兼容,因为onlyoffice在线编辑时，会将2003的文档保存为word2007的格式。
            return getContentDocx(data);
        } catch (Exception e) {
            e.printStackTrace();
            return "";
        }
    }

    /**
     * 获取正文文件内容，PDF方法
     *
     * @param data 二进制文件内容
     * @return
     */
    public static String getContentPdf(byte[] data) {
        StringBuilder result = new StringBuilder();
        try (PDDocument load = PDDocument.load(data)) {
            PDFTextStripper stripper = new PDFTextStripper();
            stripper.setSortByPosition(true);
            for (int i = 1; i < load.getNumberOfPages() + 1; i++) {
                //读取pdf的开始到结束页
                stripper.setStartPage(i);
                stripper.setEndPage(i);
                String text = stripper.getText(load);
                //拼接不同页数的数据返回
                result.append(text);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return result.toString();
    }

    /**
     * 获取正文文件内容，PPTX方法
     *
     * @param data 二进制文件内容
     * @return
     */
    public static String getContentPptx(byte[] data) {
        String result = "";
        try (InputStream is = new ByteArrayInputStream(data)) {
            XMLSlideShow ppt = new XMLSlideShow(is);
            SlideShowExtractor e = new SlideShowExtractor(ppt);
            return e.getText();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return result.replace("\n\n", "\n");
    }

    /**
     * 获取正文文件内容，PPTX方法
     *
     * @param data 二进制文件内容
     * @return
     */
    public static String getContentPpt(byte[] data) {
        try (InputStream is = new ByteArrayInputStream(data)) {
            HSLFSlideShow slideShow = new HSLFSlideShow(is);
            SlideShowExtractor e = new SlideShowExtractor(slideShow);
            return e.getText();
        } catch (IOException e) {
            e.printStackTrace();
            return "";
        }
    }

    /**
     * 获取正文文件内容，XLSX方法
     *
     * @param data 二进制文件内容
     * @return 结果
     */
    public static String getContentXlsx(byte[] data) {
        try (InputStream is = new ByteArrayInputStream(data);
             XSSFWorkbook xssfWorkbook = new XSSFWorkbook(is)) {
            //创建工作簿对象
            return getSheetText(xssfWorkbook);
        } catch (IOException e) {
            e.printStackTrace();
            return "";
        }
    }

    /**
     * 获取正文文件内容，xls
     *
     * @param data 二进制文件内容
     * @return
     */
    public static String getContentXls(byte[] data) {
        try (InputStream is = new ByteArrayInputStream(data);
             HSSFWorkbook hssfWorkbook = new HSSFWorkbook(is)) {
            //创建工作簿对象
            return getSheetText(hssfWorkbook);
        } catch (IOException e) {
            e.printStackTrace();
            return "";
        }
    }

    /**
     * 获取工作表内容
     *
     * @param workbook 工作簿
     * @return txt
     */
    private static String getSheetText(Workbook workbook) {
        String result = "";
        //获取工作簿下sheet的个数
        int sheetNum = workbook.getNumberOfSheets();
        //遍历工作簿中的所有数据
        for (int i = 0; i < sheetNum; i++) {
            //读取第i个工作表
            Sheet sheet = workbook.getSheetAt(i);
            //获取最后一行的num，即总行数。此处从0开始
            int maxRow = sheet.getLastRowNum();
            for (int rowNum = 0; rowNum <= maxRow; rowNum++) {
                Row row = sheet.getRow(rowNum);
                if (row != null) {
                    //获取最后单元格num，即总单元格数 ***注意：此处从1开始计数***
                    int maxRol = row.getLastCellNum();
                    for (int cellNum = 0; cellNum < maxRol; cellNum++) {
                        result += row.getCell(cellNum) + "  ";
                    }
                    result += "\n";
                }
            }
        }
        return result.replace("\n\n", "\n");
    }

    public static String getContentTxt(byte[] data) {
        return new String(data, StandardCharsets.UTF_8).trim();
//        try (InputStream is = new ByteArrayInputStream(data); BufferedReader br = new BufferedReader(data)) {
//            StringBuilder sb = new StringBuilder();
//            String line;
//            while ((line = br.readLine()) != null) {
//                sb.append(line).append(System.lineSeparator());
//            }
//            return sb.toString();
//        } catch (IOException e) {
//            e.printStackTrace();
//            return "";
//        }
    }

    public static void main(String[] args) throws Exception {
        String path = "D:\\SYSTEM\\Desktop\\temp\\parse\\test.docx";

        byte[] data = Files.readAllBytes(Paths.get(path));
        String result = getContentDoc(data);
        System.err.println(result);
    }
}
