package com.doc.system.service.impl;

import com.alibaba.fastjson2.JSONObject;
import com.doc.common.enums.AlarmType;
import com.doc.common.enums.EventLevel;
import com.doc.common.utils.DateUtils;
import com.doc.system.domain.SysAlarm;
import com.doc.system.domain.SysConfig;
import com.doc.system.domain.vo.AlarmVO;
import com.doc.system.mapper.SysAlarmMapper;
import com.doc.system.service.ISysAlarmService;
import com.doc.system.service.ISysAlarmSmsService;
import com.doc.system.service.ISysConfigService;
import com.doc.system.service.TencentSmsService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.File;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Date;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * 系统告警Service业务层处理
 *
 * @author wukai
 * @date 2023-11-24
 */
@Service
public class SysAlarmServiceImpl implements ISysAlarmService {
    @Resource
    private SysAlarmMapper sysAlarmMapper;
    @Resource
    private ISysConfigService sysConfigService;
    @Resource
    private ISysAlarmSmsService alarmSmsService;


    /**
     * 查询系统告警
     *
     * @param alarmId 系统告警主键
     * @return 系统告警
     */
    @Override
    public SysAlarm selectSysAlarmByAlarmId(Long alarmId) {
        return sysAlarmMapper.selectSysAlarmByAlarmId(alarmId);
    }

    /**
     * 查询系统告警列表
     *
     * @param sysAlarm 系统告警
     * @return 系统告警
     */
    @Override
    public List<SysAlarm> selectSysAlarmList(SysAlarm sysAlarm) {
        return sysAlarmMapper.selectSysAlarmList(sysAlarm);
    }

    /**
     * 新增系统告警
     *
     * @param sysAlarm 系统告警
     * @return 结果
     */
    @Override
    public int insertSysAlarm(SysAlarm sysAlarm) {
        int r = sysAlarmMapper.insertSysAlarm(sysAlarm);
        SysConfig config = sysConfigService.selectConfig("alarm.config");
        AlarmVO vo = new AlarmVO();
        try {
            vo = JSONObject.parseObject(config.getConfigValue(), AlarmVO.class);
            String open = "1";
            //短信开启状态为1
            if (!open.equals(vo.getStatus())) {
                return r;
            }

            //1.先查询是否已经发送过短信
            int type = sysAlarm.getAlarmType();
            if (alarmSmsService.isSend(type, sysAlarm.getRemark())) {
                return r;
            }
            //2.在查询是否满足发送短信的条件
            if (type == AlarmType.MONITOR_HDD.ordinal()) {
                if (Double.parseDouble(sysAlarm.getRemark()) > vo.getHdd_used()) {
                    //发送短信
                    TencentSmsService.send(TencentSmsService.PERFORMANCE_MONITORING, vo.getPhone(), "硬盘", "空间", sysAlarm.getRemark() + "%");
                }
            }
            SysAlarm search = new SysAlarm();
            search.setAlarmType(type);
            search.setAlarmTime(new Date());
            List<SysAlarm> list;
            if (type == AlarmType.IP_ILLEGAL.ordinal()) {
                search.setRemark(sysAlarm.getRemark());
                list = sysAlarmMapper.selectSysAlarmList(search);
                if (list.size() > vo.getIp_times()) {
                    //发送短信
                    TencentSmsService.send(TencentSmsService.ALARM_ILLEGAL, vo.getPhone(), DateUtils.getTime(), sysAlarm.getRemark(), "非法IP访问");
                }
            } else {
                list = sysAlarmMapper.selectSysAlarmList(search);
                AlarmVO finalVo = vo;
                AtomicInteger times = new AtomicInteger();
                list.forEach(alarm -> {
                    double d = Double.parseDouble(alarm.getRemark());
                    if (d > finalVo.getCpu_used()) {
                        times.getAndIncrement();
                    }
                });
                if (times.intValue() > finalVo.getCpu_times()) {
                    //发送短信
                    if (type == AlarmType.MONITOR_CPU.ordinal()) {
                        TencentSmsService.send(TencentSmsService.PERFORMANCE_MONITORING, vo.getPhone(), "CPU", "当前", sysAlarm.getRemark() + "%");
                    } else if (type == AlarmType.MONITOR_MEM.ordinal()) {
                        TencentSmsService.send(TencentSmsService.PERFORMANCE_MONITORING, vo.getPhone(), "内存", "当前", sysAlarm.getRemark() + "%");

                    }
                }

            }
            //3.如果满足，则发送短信

        } catch (
                Exception ignored) {
        }
        return r;
    }

    /**
     * 修改系统告警
     *
     * @param sysAlarm 系统告警
     * @return 结果
     */
    @Override
    public int updateSysAlarm(SysAlarm sysAlarm) {
        return sysAlarmMapper.updateSysAlarm(sysAlarm);
    }

    /**
     * 批量删除系统告警
     *
     * @param alarmIds 需要删除的系统告警主键
     * @return 结果
     */
    @Override
    public int deleteSysAlarmByAlarmIds(Long[] alarmIds) {
        return sysAlarmMapper.deleteSysAlarmByAlarmIds(alarmIds);
    }

    /**
     * 删除系统告警信息
     *
     * @param alarmId 系统告警主键
     * @return 结果
     */
    @Override
    public int deleteSysAlarmByAlarmId(Long alarmId) {
        return sysAlarmMapper.deleteSysAlarmByAlarmId(alarmId);
    }

    /**
     * 硬件监控
     */
    @Override
    public void monitor() {
        File[] roots = File.listRoots();
        BigDecimal total = BigDecimal.ZERO;
        BigDecimal used = BigDecimal.ZERO;
        for (File f : roots) {
            total = total.add(BigDecimal.valueOf(f.getTotalSpace()));
            used = used.add(BigDecimal.valueOf(f.getUsableSpace()));
        }

        used = used.multiply(BigDecimal.valueOf(100)).divide(total, 2, RoundingMode.HALF_UP);
        int high = 80;
        int middle = 60;
        EventLevel level = EventLevel.LOW;
        if (used.intValue() > high) {
            level = EventLevel.HIGH;
        } else if (used.intValue() > middle) {
            level = EventLevel.MIDDLE;
        }
        SysAlarm alarm = new SysAlarm();
        alarm.setAlarmType(level.ordinal());
        alarm.setAlarmContent("硬盘使用已达:" + used + "%");
        alarm.setRemark(used.toString());
        alarm.setAlarmTime(new Date());
        insertSysAlarm(alarm);
    }
}
